"""
Diverging color maps from the CartoColors schemes:
https://github.com/CartoDB/CartoColor/wiki/CARTOColor-Scheme-Names
"""
from __future__ import absolute_import

import itertools

from . import cartocolorspalette
from . import colormaps
from .. import utils

_PALETTE_TYPE = 'diverging'
_NAMES_TO_DATA = {
        'ArmyRose': colormaps._ARMYROSE,
        'Fall': colormaps._FALL,
        'Geyser': colormaps._GEYSER,
        'Temps': colormaps._TEMPS,
        'TealRose': colormaps._TEALROSE,
        'Tropic': colormaps._TROPIC,
        'Earth': colormaps._EARTH,
}
_NAME_MAP = utils.make_name_map(_NAMES_TO_DATA.keys())
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(
    sorted(_NAMES_TO_DATA.keys()),
    range(2, 8))


print_maps = utils.print_maps_factory(
    'diverging cartocolors', _NAMES_AND_LENGTHS, _PALETTE_TYPE)
get_map = utils.get_map_factory(
    'diverging cartocolors', __name__, _NAMES_TO_DATA, _PALETTE_TYPE,
    cartocolorspalette.CartoColorsMap)


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



"""
Sequential color maps from the CartoColors schemes:
https://github.com/CartoDB/CartoColor/wiki/CARTOColor-Scheme-Names
"""
from __future__ import absolute_import

import itertools

from . import cartocolorspalette
from . import colormaps
from .. import utils

_PALETTE_TYPE = 'sequential'
_NAMES_TO_DATA = {
    'Burg': colormaps._BURG,
    'BurgYl': colormaps._BURGYL,
    'RedOr': colormaps._REDOR,
    'OrYel': colormaps._ORYEL,
    'Peach': colormaps._PEACH,
    'PinkYl': colormaps._PINKYL,
    'Mint': colormaps._MINT,
    'BluGrn': colormaps._BLUGRN,
    'DarkMint': colormaps._DARKMINT,
    'Emrld': colormaps._EMRLD,
    'agGrnYl': colormaps._AGGRNYL,
    'BluYl': colormaps._BLUYL,
    'Teal': colormaps._TEAL,
    'TealGrn': colormaps._TEALGRN,
    'Purp': colormaps._PURP,
    'PurpOr': colormaps._PURPOR,
    'Sunset': colormaps._SUNSET,
    'Magenta': colormaps._MAGENTA,
    'SunsetDark': colormaps._SUNSETDARK,
    'agSunset': colormaps._AGSUNSET,
    'BrwnYl': colormaps._BRWNYL,
}
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(
    sorted(_NAMES_TO_DATA.keys()),
    range(2, 8))


print_maps = utils.print_maps_factory(
    'sequential cartocolors', _NAMES_AND_LENGTHS, _PALETTE_TYPE)
get_map = utils.get_map_factory(
    'sequential cartocolors', __name__, _NAMES_TO_DATA, _PALETTE_TYPE,
    cartocolorspalette.CartoColorsMap)


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



from __future__ import absolute_import
from __future__ import print_function
"""
Color palettes derived from http://wesandersonpalettes.tumblr.com/.

"""

import webbrowser

from ..palette import Palette


_tumblr_template = 'http://wesandersonpalettes.tumblr.com/post/{0}'
_palette_type = 'qualitative'

# Tumblr palettes in chronological order
_palettes = {
    'Chevalier': {
        'colors': [
            (53, 82, 67), (254, 202, 73), (201, 213, 213), (187, 162, 137)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79263620764/hotel-chevalier')
    },
    'Moonrise1': {
        'colors': [
            (114, 202, 221), (240, 165, 176), (140, 133, 54), (195, 180, 119),
            (250, 208, 99)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79263667140/sam-i-love-you-but-you-dont-know-what-youre')
    },
    'Mendl': {
        'colors': [
            (222, 141, 185), (184, 192, 246), (207, 147, 135), (92, 128, 204)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79348206200/mendls-heaven')
    },
    'Margot1': {
        'colors': [
            (137, 119, 18), (243, 194, 164), (246, 159, 151), (254, 214, 140),
            (98, 144, 117)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79348364517/margot-takes-a-bath')
    },
    'Cavalcanti': {
        'colors': [
            (209, 170, 0), (8, 50, 19), (146, 148, 96), (111, 152, 121),
            (132, 33, 17)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79348553036/castello-cavalcanti-how-can-i-help')
    },
    'Moonrise2': {
        'colors': [
            (102, 124, 116), (181, 106, 39), (194, 186, 124), (31, 25, 23)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79641731527/sam-why-do-you-always-use-binoculars-suzy-it')
    },
    'Margot2': {
        'colors': [
            (118, 139, 147), (188, 36, 15), (249, 236, 197), (212, 115, 41)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79641785036/margot-takes-a-break')
    },
    'Moonrise3': {
        'colors': [
            (242, 218, 82), (197, 157, 0), (203, 203, 201), (27, 30, 20)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79783357790/suzy-ive-always-wanted-to-be-an-orphan-most-of')
    },
    'GrandBudapest1': {
        'colors': [
            (238, 174, 101), (251, 79, 85), (72, 19, 19), (204, 95, 39)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79784389334/the-grand-budapest-hotel')
    },
    'Moonrise4': {
        'colors': [
            (123, 135, 97), (193, 166, 46), (79, 143, 107), (59, 69, 60),
            (159, 50, 8)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('79956897654/coming-soon')
    },
    'Zissou': {
        'colors': [
            (0, 153, 230), (18, 37, 90), (242, 56, 20), (223, 183, 139),
            (182, 195, 197)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79956949771/steve-zissou-dont-point-that-gun-at-him-hes-an')
    },
    'Royal1': {
        'colors': [
            (121, 164, 58), (242, 214, 175), (94, 72, 41), (24, 20, 1)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '79957796915/royal-o-reilly-tenenbaum-1932-2001')
    },
    'Darjeeling1': {
        'colors': [
            (158, 151, 151), (194, 142, 0), (131, 102, 89), (156, 90, 51)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '80149649946/jack-i-wonder-if-the-three-of-us-wouldve-been')
    },
    'FantasticFox1': {
        'colors': [
            (249, 219, 32), (147, 75, 78), (66, 23, 13), (194, 121, 34),
            (226, 200, 167)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '80149872170/mrs-fox-you-know-you-really-are-fantastic-mr')
    },
    'Margot3': {
        'colors': [
            (135, 162, 164), (202, 160, 101), (214, 202, 191), (214, 160, 160)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '109473707895/etheline-raleigh-says-youve-been-spending-six')
    },
    'GrandBudapest2': {
        'colors': [
            (255, 166, 142), (251, 204, 183), (140, 17, 8), (41, 11, 4)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '109473911685/m-gustave-you-see-there-are-still-faint')
    },
    'Aquatic1': {
        'colors': [
            (52, 36, 25), (28, 64, 39), (241, 201, 14), (102, 88, 153),
            (184, 147, 130)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '109568074320/steve-zissou-the-deeper-you-go-the-weirder-life')
    },
    'Darjeeling2': {
        'colors': [
            (213, 227, 216), (97, 138, 152), (249, 218, 149), (174, 75, 22),
            (120, 112, 100)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('109980167015/peter-fuck-the-itinerary')
    },
    'FantasticFox2': {
        'colors': [
            (228, 191, 68), (198, 87, 66), (154, 208, 187), (51, 39, 55),
            (171, 161, 141)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format('110716093015/ash-should-we-dance')
    },
    'GrandBudapest3': {
        'colors': [
            (255, 220, 182), (37, 56, 69), (231, 173, 157), (102, 117, 110),
            (139, 63, 49), (150, 109, 53)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '112305028860/m-gustave-mendls-is-the-best')
    },
    'Royal2': {
        'colors': [
            (194, 171, 186), (140, 59, 73), (182, 172, 166), (33, 32, 83),
            (209, 211, 213)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '115124780615/royal-anybody-interested-in-grabbing-a-couple-of')
    },
    'Moonrise5': {
        'colors': [
            (223, 140, 144), (216, 210, 142), (245, 190, 37), (61, 74, 28),
            (209, 48, 96), (168, 107, 76)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '116909186645/walt-bishop-our-daughters-been-abducted-by-one')
    },
    'GrandBudapest4': {
        'colors': [
            (186, 110, 0), (80, 32, 86), (255, 246, 187), (154, 127, 25),
            (31, 18, 27)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '117849683385/concierge-and-how-long-will-you-be-staying-with')
    },
    'Moonrise6': {
        'colors': [
            (191, 81, 61), (201, 162, 150), (197, 193, 136), (123, 177, 145),
            (217, 174, 48)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '118877161325/sam-im-not-that-strong-of-a-swimmer-so-i-wear-a')
    },
    'GrandBudapest5': {
        'colors': [
            (225, 146, 131), (140, 27, 76), (209, 147, 54), (231, 199, 190),
            (51, 12, 0)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '122169507295/m-gustave-its-quite-a-thing-winning-the-loyalty')
    },
    'Aquatic2': {
        'colors': [
            (139, 156, 184), (233, 229, 65), (88, 159, 194), (160, 141, 94),
            (189, 185, 206)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '125170837755/steve-zissou-please-dont-make-fun-of-me-i-just')
    },
    'Royal3': {
        'colors': [
            (252, 87, 108), (237, 126, 83), (226, 153, 139), (46, 23, 24),
            (251, 214, 202)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '129921576355/royal-ive-always-been-considered-an-asshole-for')
    },
    'Moonrise7': {
        'colors': [
            (35, 35, 85), (97, 104, 96), (167, 91, 68), (145, 43, 41),
            (255, 227, 143)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '137096576550/social-services-access-denied')
    },
    'Aquatic3': {
        'colors': [
            (214, 161, 66), (194, 128, 114), (200, 183, 161), (189, 68, 45),
            (100, 84, 60)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '139482629630/ah-weve-never-made-great-husbands-have-we-of')
    },
    'Darjeeling3': {
        'colors': [
            (168, 171, 80), (255, 232, 61), (169, 211, 210), (36, 71, 125),
            (90, 145, 124)
        ],
        'type': _palette_type,
        'url': _tumblr_template.format(
            '143938510215/the-chief-steward-welcome-aboard')
    }
}

_map_names = {}
for k in _palettes:
    _map_names[k.lower()] = k


class WesAndersonMap(Palette):
    """
    Representation of a color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    map_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.
    url : str
        URL on the web where this color map can be viewed.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap
    wap_url : str
        URL on the web where this color map can be viewed.

    """
    def __init__(self, name, map_type, colors, url):
        super(WesAndersonMap, self).__init__(name, map_type, colors)
        self.url = url

    def wap(self):
        """
        View this color palette on the web.
        Will open a new tab in your web browser.

        """
        webbrowser.open_new_tab(self.url)  # pragma: no cover


def print_maps():
    """
    Print a list of Wes Anderson palettes.

    """
    namelen = max(len(k) for k in _palettes)
    fmt = '{0:' + str(namelen + 4) + '}{1:16}{2:}'

    for k in sorted(_palettes.keys()):
        print(fmt.format(k, _palettes[k]['type'], len(_palettes[k]['colors'])))


def get_map(name, reverse=False):
    """
    Get a Wes Anderson palette by name.

    Parameters
    ----------
    name : str
        Name of map. Use palettable.wesanderson.print_maps
        to see available names.
    reverse : bool, optional
        If True reverse colors from their default order.

    Returns
    -------
    palette : WesAndersonMap

    """
    name = _map_names[name.lower()]
    palette = _palettes[name]

    if reverse:
        name += '_r'
        palette['colors'] = list(reversed(palette['colors']))

    return WesAndersonMap(
        name, palette['type'], palette['colors'], palette['url'])


def _get_all_maps():
    """
    Returns a dictionary of all Wes Anderson palettes,
    including reversed ones.

    """
    fmt = '{0}_{1}'.format
    d = dict(
        (fmt(name, m.number), m)
        for name, m in ((name, get_map(name)) for name in _palettes))
    fmt = '{0}_{1}_r'.format
    d.update(
        dict(
            (fmt(name, m.number), m)
            for name, m in (
                (name, get_map(name, reverse=True)) for name in _palettes)))
    return d



"""
Palettable is a pure Python package for accessing
a variety of color maps from Python, including colorbrewer2,
Tableau, and whimsical Wes Anderson maps.

"""
from __future__ import absolute_import

from . import cmocean
from . import cartocolors
from . import colorbrewer
from . import cubehelix
from . import lightbartlein
from . import matplotlib
from . import mycarta
from . import wesanderson
from . import tableau

version = __version__ = '3.2.0'



from __future__ import absolute_import, print_function
"""
New matplotlib color palettes as described at https://bids.github.io/colormap

"""

import itertools

from . import colormaps
from .. import utils
from ..palette import Palette

_PALETTE_TYPE = 'sequential'
_NAMES_TO_DATA = {
    'Magma': colormaps._MAGMA_DATA,
    'Inferno': colormaps._INFERNO_DATA,
    'Plasma': colormaps._PLASMA_DATA,
    'Viridis': colormaps._VIRIDIS_DATA
}
_NAME_MAP = utils.make_name_map(_NAMES_TO_DATA.keys())
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(
    sorted(_NAMES_TO_DATA.keys()))


class MatplotlibMap(Palette):
    """
    Representation of a color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    palette_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap
    url : str
        Website with related info.

    """
    url = 'https://bids.github.io/colormap'

    def __init__(self, name, palette_type, colors):
        super(MatplotlibMap, self).__init__(name, palette_type, colors)


print_maps = utils.print_maps_factory(
    'matplotlib', _NAMES_AND_LENGTHS, _PALETTE_TYPE)
get_map = utils.get_map_factory(
    'matplotlib', __name__, _NAMES_TO_DATA, _PALETTE_TYPE,
    MatplotlibMap)



from __future__ import absolute_import, print_function
"""
Sequential Light & Bartlein palettes as described at
http://geog.uoregon.edu/datagraphics/color_scales.htm

"""
import itertools as it

from . import colordata
from . import lightbartlein
from .. import utils

_PALETTE_TYPE = 'sequential'
_NAMES_TO_DATA = {
    'Blues7': colordata._BLUE_7,
    'Blues10': colordata._BLUE_10,
}
_MAX_LENS = {
    'Blues7': 7,
    'Blues10': 10,
}
_NAMES_AND_LENGTHS = list(
    it.chain.from_iterable(
        utils.make_names_and_lengths([name], range(2, _MAX_LENS[name] + 1))
        for name in sorted(_NAMES_TO_DATA)
    )
)

print_maps = utils.print_maps_factory('sequential Light & Bartlein',
                                      _NAMES_AND_LENGTHS,
                                      _PALETTE_TYPE)

get_map = utils.get_map_factory('sequential Light & Bartlein',
                                __name__,
                                _NAMES_TO_DATA,
                                _PALETTE_TYPE,
                                lightbartlein.LightBartleinMap,
                                is_evenly_spaced=True)

globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



# coding: utf-8

from __future__ import absolute_import

import sys

try:
    import matplotlib.pyplot as plt
    from matplotlib.colors import (
        ListedColormap, BoundaryNorm, Normalize, LinearSegmentedColormap)
    from matplotlib.colorbar import ColorbarBase
except ImportError:     # pragma: no cover
    HAVE_MPL = False
except RuntimeError:  # e.g. OS X, if Python is not installed as a framework
    HAVE_MPL = False
else:
    HAVE_MPL = True


class Palette(object):
    """
    Representation of a color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    map_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap

    """
    def __init__(self, name, map_type, colors):
        self.name = name
        self.type = map_type
        self.number = len(colors)
        self.colors = colors

    @property
    def hex_colors(self):
        """
        Colors as a tuple of hex strings. (e.g. '#A912F4')

        """
        hc = []

        for color in self.colors:
            h = '#' + ''.join('{0:>02}'.format(hex(c)[2:].upper())
                              for c in color)
            hc.append(h)

        return hc

    @property
    def mpl_colors(self):
        """
        Colors expressed on the range 0-1 as used by matplotlib.

        """
        mc = []

        for color in self.colors:
            mc.append(tuple([x / 255. for x in color]))

        return mc

    @property
    def mpl_colormap(self):
        """
        A basic matplotlib color map. If you want to specify keyword arguments
        use the `get_mpl_colormap` method.

        """
        return self.get_mpl_colormap()

    def get_mpl_colormap(self, **kwargs):
        """
        A color map that can be used in matplotlib plots. Requires matplotlib
        to be importable. Keyword arguments are passed to
        `matplotlib.colors.LinearSegmentedColormap.from_list`.

        """
        if not HAVE_MPL:    # pragma: no cover
            raise RuntimeError('matplotlib not available.')

        cmap = LinearSegmentedColormap.from_list(self.name,
                                                 self.mpl_colors, **kwargs)

        return cmap

    def show_as_blocks(self, block_size=100):
        """
        Show colors in the IPython Notebook using ipythonblocks.

        Parameters
        ----------
        block_size : int, optional
            Size of displayed blocks.

        """
        from ipythonblocks import BlockGrid

        grid = BlockGrid(self.number, 1, block_size=block_size)

        for block, color in zip(grid, self.colors):
            block.rgb = color

        grid.show()

    def _write_image(self, fp, style, format='png', size=(6, 1)):
        """
        Write the color map as an image to a file-like object.

        Parameters
        ----------
        fp : file-like
            A file-like object such as an open file pointer or
            a StringIO/BytesIO instance.
        style : {'discrete', 'continuous'}
            Whether to make the color map image as a discrete map
            or a continuous map.
        format : str, optional
            An image format that will be understood by matplotlib.
        size : tuple of int, optional
            (width, height) of image to make in units of inches.

        """
        if not HAVE_MPL:    # pragma: no cover
            raise RuntimeError('matplotlib not available.')

        fig = plt.figure(figsize=size, frameon=False)
        ax = fig.add_axes([0, 0, 1, 1], frameon=False)
        ax.set_axis_off()

        if style == 'discrete':
            # make a bounded color map showing only the defined colors
            ncolors = self.number
            norm = BoundaryNorm(range(ncolors + 1), ncolors=ncolors)
            cmap = ListedColormap(self.mpl_colors)

        elif style == 'continuous':
            # make the smooth, interpolated color map
            cmap = self.mpl_colormap
            norm = Normalize(vmin=0, vmax=1)

        ColorbarBase(ax, cmap=cmap, norm=norm, orientation='horizontal')
        fig.savefig(fp, format=format)
        plt.close(fig)

    def show_discrete_image(self, size=(6, 1)):
        """
        Embed an image of this discrete color map in the IPython Notebook.

        Parameters
        ----------
        size : tuple of int, optional
            (width, height) of image to make in units of inches.

        """
        if sys.version_info[0] == 2:
            from StringIO import StringIO as BytesIO
        elif sys.version_info[0] == 3:
            from io import BytesIO

        from IPython.display import display
        from IPython.display import Image as ipyImage

        im = BytesIO()
        self._write_image(im, 'discrete', format='png', size=size)
        display(ipyImage(data=im.getvalue(), format='png'))

    def save_discrete_image(self, filename, size=(6, 1), format=None):
        """
        Save an image of this discrete color map to a file.

        Parameters
        ----------
        filename : str
            If `format` is None the format will be inferred from the
            `filename` extension.
        size : tuple of int, optional
            (width, height) of image to make in units of inches.
        format : str, optional
            An image format that will be understood by matplotlib.

        """
        with open(filename, 'wb') as f:
            self._write_image(
                f, 'discrete', format=filename.split('.')[-1], size=size)

    def show_continuous_image(self, size=(6, 1)):
        """
        Embed an image of this continuous color map in the IPython Notebook.

        Parameters
        ----------
        size : tuple of int, optional
            (width, height) of image to make in units of inches.

        """
        if sys.version_info[0] == 2:
            from StringIO import StringIO as BytesIO
        elif sys.version_info[0] == 3:
            from io import BytesIO

        from IPython.display import display
        from IPython.display import Image as ipyImage

        im = BytesIO()
        self._write_image(im, 'continuous', format='png', size=size)
        display(ipyImage(data=im.getvalue(), format='png'))

    def save_continuous_image(self, filename, size=(6, 1), format=None):
        """
        Save an image of this continuous color map to a file.

        Parameters
        ----------
        filename : str
            If `format` is None the format will be inferred from the
            `filename` extension.
        size : tuple of int, optional
            (width, height) of image to make in units of inches.
        format : str, optional
            An image format that will be understood by matplotlib.

        """
        with open(filename, 'wb') as f:
            self._write_image(
                f, 'continuous', format=filename.split('.')[-1], size=size)



print("import: 'palettable'")
import palettable

print("import: 'palettable.cmocean'")
import palettable.cmocean

print("import: 'palettable.colorbrewer'")
import palettable.colorbrewer

print("import: 'palettable.cubehelix'")
import palettable.cubehelix

print("import: 'palettable.matplotlib'")
import palettable.matplotlib

print("import: 'palettable.mycarta'")
import palettable.mycarta

print("import: 'palettable.tableau'")
import palettable.tableau

print("import: 'palettable.wesanderson'")
import palettable.wesanderson




from __future__ import absolute_import

from .. import utils
from .mycarta import __doc__, print_maps, get_map, _NAMES_AND_LENGTHS


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



from __future__ import absolute_import
from __future__ import print_function

import json
import webbrowser

from pkg_resources import resource_string

from ..palette import Palette

__all__ = ('COLOR_MAPS', 'print_maps', 'print_all_maps', 'print_maps_by_type',
           'get_map', 'MAP_TYPES', 'BrewerMap')

f = resource_string(__name__, 'data/colorbrewer_all_schemes.json')
COLOR_MAPS = json.loads(f.decode('ascii'))

MAP_TYPES = ('sequential', 'diverging', 'qualitative')


def print_maps(map_type=None, number=None):
    """
    Print maps by type and/or number of defined colors.

    Parameters
    ----------
    map_type : {'sequential', 'diverging', 'qualitative'}, optional
        Filter output by map type. By default all maps are printed.
    number : int, optional
        Filter output by number of defined colors. By default there is
        no numeric filtering.

    """
    if not map_type and not number:
        print_all_maps()

    elif map_type:
        print_maps_by_type(map_type, number)

    else:
        s = ('Invalid parameter combination. '
             'number without map_type is not supported.')
        raise ValueError(s)


def print_all_maps():
    """
    Print the name and number of defined colors of all available color maps.

    """
    for t in MAP_TYPES:
        print_maps_by_type(t)


def print_maps_by_type(map_type, number=None):
    """
    Print all available maps of a given type.

    Parameters
    ----------
    map_type : {'sequential', 'diverging', 'qualitative'}
        Select map type to print.
    number : int, optional
        Filter output by number of defined colors. By default there is
        no numeric filtering.

    """
    if map_type.lower() not in MAP_TYPES:
        s = 'Invalid map type, must be one of {0}'.format(MAP_TYPES)
        raise ValueError(s)

    print(map_type)

    # maps are keyed by capitalized types in COLOR_MAPS
    map_type = map_type.capitalize()
    map_keys = sorted(COLOR_MAPS[map_type].keys())

    format_str = '{0:8}  :  {1}'

    for mk in map_keys:
        num_keys = sorted(COLOR_MAPS[map_type][mk].keys(), key=int)

        if not number or str(number) in num_keys:
            num_str = '{' + ', '.join(num_keys) + '}'
            print(format_str.format(mk, num_str))


class BrewerMap(Palette):
    """
    Representation of a colorbrewer2 color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    map_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    colorbrewer2_url : str
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap

    """
    @property
    def colorbrewer2_url(self):
        """
        URL that can be used to view the color map at colorbrewer2.org.

        """
        url = 'http://colorbrewer2.org/index.html?type={0}&scheme={1}&n={2}'
        return url.format(self.type.lower(), self.name, self.number)

    def colorbrewer2(self):
        """
        View this color map at colorbrewer2.org. This will open
        colorbrewer2.org in your default web browser.

        """
        webbrowser.open_new_tab(self.colorbrewer2_url)  # pragma: no cover


def get_map(name, map_type, number, reverse=False):
    """
    Return a `BrewerMap` representation of the specified color map.

    Parameters
    ----------
    name : str
        Name of color map. Use `print_maps` to see available color maps.
    map_type : {'sequential', 'diverging', 'qualitative'}
        Select color map type.
    number : int
        Number of defined colors in color map.
    reverse : bool, optional
        Set to True to get the reversed color map.

    """
    number = str(number)

    # check for valid type
    if map_type.lower() not in MAP_TYPES:
        s = 'Invalid map type, must be one of {0}'.format(MAP_TYPES)
        raise ValueError(s)

    # maps are keyed by capitalized types in COLOR_MAPS
    map_type = map_type.capitalize()

    # make a dict of lower case map name to map name so this can be
    # insensitive to case.
    # this would be a perfect spot for a dict comprehension but going to
    # wait on that to preserve 2.6 compatibility.
    # map_names = {k.lower(): k for k in COLOR_MAPS[map_type].iterkeys()}
    map_names = dict((k.lower(), k) for k in COLOR_MAPS[map_type].keys())

    # check for valid name
    if name.lower() not in map_names:
        s = 'Invalid color map name {0!r} for type {1!r}.\n'
        s = s.format(name, map_type)
        valid_names = [str(k) for k in COLOR_MAPS[map_type].keys()]
        valid_names.sort()
        s += 'Valid names are: {0}'.format(valid_names)
        raise ValueError(s)

    name = map_names[name.lower()]

    # check for valid number
    if number not in COLOR_MAPS[map_type][name]:
        s = 'Invalid number for map type {0!r} and name {1!r}.\n'
        s = s.format(map_type, str(name))
        valid_numbers = [int(k) for k in COLOR_MAPS[map_type][name].keys()]
        valid_numbers.sort()
        s += 'Valid numbers are : {0}'.format(valid_numbers)
        raise ValueError(s)

    colors = COLOR_MAPS[map_type][name][number]['Colors']

    if reverse:
        name += '_r'
        colors = [x for x in reversed(colors)]

    return BrewerMap(name, map_type.lower(), colors)


def _load_maps_by_type(map_type):
    """
    Load all maps of a given type into a dictionary.

    Color maps are loaded as BrewerMap objects. Dictionary is
    keyed by map name and then integer numbers of defined
    colors. There is an additional 'max' key that points to the
    color map with the largest number of defined colors.

    Parameters
    ----------
    map_type : {'sequential', 'diverging', 'qualitative'}

    Returns
    -------
    maps : dict of BrewerMap

    """
    map_type = map_type.capitalize()
    seq_maps = COLOR_MAPS[map_type]

    loaded_maps = {}

    for map_name in seq_maps:
        for num in seq_maps[map_name]:
            inum = int(num)
            name = '{0}_{1}'.format(map_name, num)
            loaded_maps[name] = get_map(map_name, map_type, inum)
            loaded_maps[name + '_r'] = get_map(
                map_name, map_type, inum, reverse=True)

    return loaded_maps



from __future__ import absolute_import, print_function
"""
Diverging Light & Bartlein palettes as described at
http://geog.uoregon.edu/datagraphics/color_scales.htm

"""
import itertools as it

from . import colordata
from . import lightbartlein
from .. import utils

_PALETTE_TYPE = 'diverging'
_NAMES_TO_DATA = {
    'BlueDarkOrange12': colordata._BLUE_TO_DARK_ORANGE_12,
    'BlueDarkOrange18': colordata._BLUE_TO_DARK_ORANGE_18,
    'BlueDarkRed12': colordata._BLUE_TO_DARK_RED_12,
    'BlueDarkRed18': colordata._BLUE_TO_DARK_RED_18,
    'BlueGreen': colordata._BLUE_TO_GREEN,
    'BlueGrey': colordata._BLUE_TO_GREY,
    'BlueGray': colordata._BLUE_TO_GREY,
    'BlueOrange8': colordata._BLUE_TO_ORANGE_8,
    'BlueOrange10': colordata._BLUE_TO_ORANGE_10,
    'BlueOrange12': colordata._BLUE_TO_ORANGE_12,
    'BlueOrangeRed': colordata._BLUE_ORANGE_RED,
    'BrownBlue10': colordata._BROWN_TO_BLUE_10,
    'BrownBlue12': colordata._BROWN_TO_BLUE_12,
    'GreenMagenta': colordata._GREEN_TO_MAGENTA,
    'RedYellowBlue': colordata._RED_YELLOW_BLUE,
}
_MAX_LENS = {
    'BlueDarkOrange12': 12,
    'BlueDarkOrange18': 18,
    'BlueDarkRed12': 12,
    'BlueDarkRed18': 18,
    'BlueGreen': 14,
    'BlueGrey': 8,
    'BlueGray': 8,
    'BlueOrange8': 8,
    'BlueOrange10': 10,
    'BlueOrange12': 12,
    'BlueOrangeRed': 14,
    'BrownBlue10': 10,
    'BrownBlue12': 12,
    'GreenMagenta': 16,
    'RedYellowBlue': 11,
}
_NAMES_AND_LENGTHS = list(
    it.chain.from_iterable(
        utils.make_names_and_lengths([name], range(2, _MAX_LENS[name] + 1))
        for name in sorted(_NAMES_TO_DATA)
    )
)

print_maps = utils.print_maps_factory('diverging Light & Bartlein',
                                      _NAMES_AND_LENGTHS,
                                      _PALETTE_TYPE)

get_map = utils.get_map_factory('diverging Light & Bartlein',
                                __name__,
                                _NAMES_TO_DATA,
                                _PALETTE_TYPE,
                                lightbartlein.LightBartleinMap,
                                is_evenly_spaced=True)

globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



"""
Sequential color maps from the cmocean package:
https://github.com/matplotlib/cmocean.

"""
from __future__ import absolute_import

import itertools

from . import cmoceanpalette
from . import colormaps
from .. import utils

_PALETTE_TYPE = 'sequential'
_NAMES_TO_DATA = {
    'Algae': colormaps._ALGAE,
    'Amp': colormaps._AMP,
    'Deep': colormaps._DEEP,
    'Dense': colormaps._DENSE,
    'Gray': colormaps._GRAY,
    'Haline': colormaps._HALINE,
    'Ice': colormaps._ICE,
    'Matter': colormaps._MATTER,
    'Oxy': colormaps._OXY,
    'Phase': colormaps._PHASE,
    'Solar': colormaps._SOLAR,
    'Speed': colormaps._SPEED,
    'Tempo': colormaps._TEMPO,
    'Thermal': colormaps._THERMAL,
    'Turbid': colormaps._TURBID,
}
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(
    sorted(_NAMES_TO_DATA.keys()))


print_maps = utils.print_maps_factory(
    'sequential cmocean', _NAMES_AND_LENGTHS, _PALETTE_TYPE)
get_map = utils.get_map_factory(
    'sequential cmocean', __name__, _NAMES_TO_DATA, _PALETTE_TYPE,
    cmoceanpalette.CmoceanMap)


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



"""
Diverging color maps from the cmocean package:
https://github.com/matplotlib/cmocean.

"""
from __future__ import absolute_import

import itertools

from . import cmoceanpalette
from . import colormaps
from .. import utils

_PALETTE_TYPE = 'diverging'
_NAMES_TO_DATA = {
    'Balance': colormaps._BALANCE,
    'Curl': colormaps._CURL,
    'Delta': colormaps._DELTA,
}
_NAME_MAP = utils.make_name_map(_NAMES_TO_DATA.keys())
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(
    sorted(_NAMES_TO_DATA.keys()))


print_maps = utils.print_maps_factory(
    'diverging cmocean', _NAMES_AND_LENGTHS, _PALETTE_TYPE)
get_map = utils.get_map_factory(
    'divergine cmocean', __name__, _NAMES_TO_DATA, _PALETTE_TYPE,
    cmoceanpalette.CmoceanMap)


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))



# coding: utf-8
"""
Cubehelix color maps and palettes.

The Cubehelix algorithm makes color scales with monotonic changes in perceived
brightness. This means that a cubehelix color map gracefully degrades into
a monotonic grayscale color map when rendered without color.

Cubehelix maps are generated algorithmically, giving the user flexibility
in desiging a color map that is also safe for grayscale printers. This
module provides several cubehelix realizations, while also exposing the
algorithm directly through the :class:`Cubehelix` class.

Cubehelix was developed by `D.A Green, 2011, BASI, 39, 289
<http://adsabs.harvard.edu/abs/2011arXiv1108.5083G>`_. The original Python
port was done by James R. A. Davenport (see License).

Original License
----------------

Copyright (c) 2014, James R. A. Davenport and contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
from __future__ import absolute_import, print_function

try:
    import numpy as np
except ImportError:  # pragma: no cover
    HAVE_NPY = False
else:
    HAVE_NPY = True

from ..palette import Palette


url = 'http://adsabs.harvard.edu/abs/2011arXiv1108.5083G'
palette_type = 'sequential'


palette_names = [
    'classic_16',
    'perceptual_rainbow_16',
    'purple_16',
    'jim_special_16',
    'red_16',
    'cubehelix1_16',
    'cubehelix2_16',
    'cubehelix3_16'
]


palette_rgb = dict((
    ('classic_16',
     # dict(start=0.5, rotation=-1.5, gamma=1.0, sat=1.2,
     #      min_light=0., max_light=1., n=16)
     [[0, 0, 0],
      [22, 10, 34],
      [24, 32, 68],
      [16, 62, 83],
      [14, 94, 74],
      [35, 116, 51],
      [80, 125, 35],
      [138, 122, 45],
      [190, 117, 85],
      [218, 121, 145],
      [219, 138, 203],
      [204, 167, 240],
      [191, 201, 251],
      [195, 229, 244],
      [220, 246, 239],
      [255, 255, 255]]),
    ('perceptual_rainbow_16',
     # Similar to Matteo Niccoli's Perceptual Rainbow:
     # http://mycarta.wordpress.com/2013/02/21/perceptual-rainbow-palette-the-method/
     # https://github.com/jradavenport/cubehelix
     # dict(start_hue=240., end_hue=-300., min_sat=1., max_sat=2.5,
     #      min_light=0.3, max_light=0.8, gamma=.9, n=16)
     [[135, 59, 97],
      [143, 64, 127],
      [143, 72, 157],
      [135, 85, 185],
      [121, 102, 207],
      [103, 123, 220],
      [84, 146, 223],
      [69, 170, 215],
      [59, 192, 197],
      [60, 210, 172],
      [71, 223, 145],
      [93, 229, 120],
      [124, 231, 103],
      [161, 227, 95],
      [198, 220, 100],
      [233, 213, 117]]),
    ('purple_16',
     # dict(start=0., rotation=0.0, n=16)
     [[0, 0, 0],
      [15, 14, 35],
      [31, 28, 68],
      [47, 43, 99],
      [63, 59, 127],
      [79, 75, 152],
      [96, 91, 174],
      [113, 107, 194],
      [130, 124, 211],
      [147, 142, 225],
      [164, 160, 237],
      [182, 178, 246],
      [200, 196, 252],
      [218, 215, 255],
      [236, 235, 255],
      [255, 255, 255]]),
    ('jim_special_16',
     # http://www.ifweassume.com/2014/04/cubehelix-colormap-for-python.html
     # dict(start=0.3, rotation=-0.5, n=16)
     [[0, 0, 0],
      [22, 10, 34],
      [37, 25, 68],
      [47, 43, 99],
      [52, 65, 125],
      [55, 88, 146],
      [59, 112, 160],
      [64, 137, 169],
      [74, 160, 173],
      [89, 181, 175],
      [109, 199, 177],
      [134, 214, 180],
      [163, 227, 189],
      [195, 237, 203],
      [226, 246, 225],
      [255, 255, 255]]),
    ('red_16',
     # http://www.ifweassume.com/2014/04/cubehelix-colormap-for-python.html
     # dict(start=0., rotation=0.5, n=16)
     [[0, 0, 0],
      [19, 12, 35],
      [44, 22, 65],
      [73, 32, 90],
      [104, 41, 107],
      [134, 53, 118],
      [162, 67, 124],
      [185, 83, 126],
      [204, 102, 128],
      [216, 124, 130],
      [225, 148, 136],
      [229, 172, 147],
      [232, 196, 164],
      [236, 219, 189],
      [242, 238, 219],
      [255, 255, 255]]),
    ('cubehelix1_16',
     # http://nbviewer.ipython.org/gist/anonymous/a4fa0adb08f9e9ea4f94
     # dict(gamma=1.0, start=1.5, rotation=-1.0, sat=1.5, n=16)
     [[0, 0, 0],
      [27, 15, 0],
      [65, 23, 4],
      [104, 27, 32],
      [133, 33, 75],
      [147, 45, 126],
      [144, 66, 175],
      [129, 96, 210],
      [111, 131, 227],
      [99, 166, 226],
      [101, 197, 211],
      [120, 219, 194],
      [153, 233, 185],
      [193, 240, 191],
      [230, 245, 216],
      [255, 255, 255]]),
    ('cubehelix2_16',
     # http://nbviewer.ipython.org/gist/anonymous/a4fa0adb08f9e9ea4f94
     # dict(gamma=1.0, start=2.0, rotation=1.0, sat=1.5, n=16)
     [[0, 0, 0],
      [0, 28, 14],
      [0, 51, 47],
      [7, 65, 91],
      [35, 71, 135],
      [78, 72, 168],
      [129, 72, 184],
      [177, 77, 181],
      [214, 90, 165],
      [235, 113, 143],
      [238, 142, 128],
      [230, 175, 127],
      [219, 206, 144],
      [216, 231, 178],
      [226, 247, 219],
      [255, 255, 255]]),
    ('cubehelix3_16',
     # http://nbviewer.ipython.org/gist/anonymous/a4fa0adb08f9e9ea4f94
     # dict(gamma=1.0, start=2.0, rotation=1.0, sat=3, n=16)
     [[0, 0, 0],
      [0, 39, 12],
      [0, 68, 60],
      [0, 80, 131],
      [3, 75, 202],
      [72, 60, 252],
      [156, 43, 255],
      [235, 36, 244],
      [255, 45, 194],
      [255, 73, 134],
      [255, 115, 86],
      [255, 164, 67],
      [235, 209, 85],
      [211, 241, 135],
      [215, 255, 200],
      [255, 255, 255]]),
))


class Cubehelix(Palette):
    """
    Representation of a Cubehelix color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    colors : list
        Colors as list of 0-255 RGB triplets.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap

    """
    url = url

    def __init__(self, name, colors):
        super(Cubehelix, self).__init__(name, palette_type, colors)

    @classmethod
    def make(cls, start=0.5, rotation=-1.5, gamma=1.0,
             start_hue=None, end_hue=None,
             sat=None, min_sat=1.2, max_sat=1.2,
             min_light=0., max_light=1.,
             n=256., reverse=False, name='custom_cubehelix'):
        """
        Create an arbitrary Cubehelix color palette from the algorithm.

        See http://adsabs.harvard.edu/abs/2011arXiv1108.5083G for a technical
        explanation of the algorithm.

        Parameters
        ----------
        start : scalar, optional
            Sets the starting position in the RGB color space. 0=blue, 1=red,
            2=green. Default is ``0.5`` (purple).
        rotation : scalar, optional
            The number of rotations through the rainbow. Can be positive
            or negative, indicating direction of rainbow. Negative values
            correspond to Blue->Red direction. Default is ``-1.5``.
        start_hue : scalar, optional
            Sets the starting color, ranging from [-360, 360]. Combined with
            `end_hue`, this parameter overrides ``start`` and ``rotation``.
            This parameter is based on the D3 implementation by @mbostock.
            Default is ``None``.
        end_hue : scalar, optional
            Sets the ending color, ranging from [-360, 360]. Combined with
            `start_hue`, this parameter overrides ``start`` and ``rotation``.
            This parameter is based on the D3 implementation by @mbostock.
            Default is ``None``.
        gamma : scalar, optional
            The gamma correction for intensity. Values of ``gamma < 1``
            emphasize low intensities while ``gamma > 1`` emphasises high
            intensities. Default is ``1.0``.
        sat : scalar, optional
            The uniform saturation intensity factor. ``sat=0`` produces
            grayscale, while ``sat=1`` retains the full saturation. Setting
            ``sat>1`` oversaturates the color map, at the risk of clipping
            the color scale. Note that ``sat`` overrides both ``min_stat``
            and ``max_sat`` if set.
        min_sat : scalar, optional
            Saturation at the minimum level. Default is ``1.2``.
        max_sat : scalar, optional
            Satuation at the maximum level. Default is ``1.2``.
        min_light : scalar, optional
            Minimum lightness value. Default is ``0``.
        max_light : scalar, optional
            Maximum lightness value. Default is ``1``.
        n : scalar, optional
            Number of discrete rendered colors. Default is ``256``.
        reverse : bool, optional
            Set to ``True`` to reverse the color map. Will go from black to
            white. Good for density plots where shade -> density.
            Default is ``False``.
        name : str, optional
            Name of the color map (defaults to ``'custom_cubehelix'``).

        Returns
        -------
        palette : `Cubehelix`
            A Cubehelix color palette.
        """
        if not HAVE_NPY:  # pragma: no cover
            raise RuntimeError('numpy not available.')
        # start_hue/end_hue were popularized by D3's implementation
        # and will override start/rotation if set
        if start_hue is not None and end_hue is not None:
            start = (start_hue / 360. - 1.) * 3.
            rotation = end_hue / 360. - start / 3. - 1.

        # lambd is effectively the color map grid
        lambd = np.linspace(min_light, max_light, n)

        # apply the gamma correction
        lambd_gamma = lambd ** gamma

        # Rotation angle
        # NOTE the equation for phi in Green 2011 does not have an extra `+1`
        # but the Fortran code does, as does the original cubehelix.py
        # I'm leaving out the +1 to keep to the original equation, but
        # worth investigating. In practice I see no difference!
        phi = 2.0 * np.pi * (start / 3.0 + rotation * lambd)

        if sat is None:
            sat = np.linspace(min_sat, max_sat, n)

        # Amplitude of helix from grayscale map
        amp = sat * lambd_gamma * (1. - lambd_gamma) / 2.

        # Compute the RGB vectors according to Green 2011 Eq 2
        rot_matrix = np.array([[-0.14861, +1.78277],
                               [-0.29227, -0.90649],
                               [+1.97294, 0.0]])
        sin_cos = np.array([np.cos(phi), np.sin(phi)])
        rgb = (lambd_gamma + amp * np.dot(rot_matrix, sin_cos)).T * 255.

        # Clipping is necessary in some cases when sat > 1
        np.clip(rgb, 0., 255., out=rgb)

        if reverse:
            rgb = rgb[::-1, :]

        colors = rgb.astype(int).tolist()
        return cls(name, colors)


def print_maps():
    """
    Print a list of pre-made Cubehelix palettes.

    """
    namelen = max(len(name) for name in palette_names)
    fmt = '{0:' + str(namelen + 4) + '}{1:16}'

    for name in palette_names:
        print(fmt.format(name, palette_type))


def get_map(name, reverse=False):
    """
    Get a pre-made Cubehelix palette by name.

    Parameters
    ----------
    name : str
        Name of map. Use `print_maps()` to see available names.
    reverse : bool, optional
        If True reverse colors from their default order.

    Returns
    -------
    palette : Cubehelix

    """
    try:
        # Make everthing lower case for matching
        index = [s.lower() for s in palette_names].index(name.lower())
    except ValueError:
        msg = "{0!r} is an unknown Cubehelix palette."
        raise KeyError(msg.format(name))

    real_name = palette_names[index]
    colors = palette_rgb[real_name]

    if reverse:
        real_name = real_name + '_r'
        colors = list(reversed(colors))

    return Cubehelix(real_name, colors)


def _get_all_maps():
    """
    Returns a dictionary of all Cubehelix palettes, including reversed ones.

    These default palettes are rendered with 16 colours.

    """
    d = {}
    for name in palette_names:
        d[name] = get_map(name)
        d[name + '_r'] = get_map(name, reverse=True)
    return d



"""
Palette class and utilities for CartoColors palettes.

"""
from __future__ import absolute_import

from ..palette import Palette


class CartoColorsMap(Palette):
    """
    Representation of a color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    palette_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.

    Attributes
    ----------
    name : str
    type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap
    url : str
        Website with related info.

    """
    url = 'https://github.com/CartoDB/CartoColor/wiki/CARTOColor-Scheme-Names'

    def __init__(self, name, palette_type, colors):
        super(CartoColorsMap, self).__init__(name, palette_type, colors)



from __future__ import absolute_import, division

import itertools
import math
import sys
import textwrap


def round_as_py3(value):
    """
    Implement round behaviour consistent with Python 3 for use in Python 2.
    (Such that halves are rounded toward the even side.
    Only for positive numbers.)
    Adapted from https://github.com/python/cpython/blob/6b678aea1ca5c3c3728cd5a7a6eb112b2dad8553/Python/pytime.c#L71

    """
    if math.fmod(value, 1) == 0.5:
        return 2 * round(value / 2)
    else:
        return round(value)

if sys.version_info[0] >= 3:
    round_ = round
else:
    round_ = round_as_py3


def n_to_indices(num, length):
    """
    Calculate `num` evenly spaced indices for a sequence of length `length`.

    Should be used with num >= 2, result will always include first and last.

    Parameters
    ----------
    num : int
    length : int

    Returns
    -------
    list of int

    """
    if num < 2:
        raise ValueError('num must be 2 or larger, got {0}'.format(num))
    elif num > length:
        raise ValueError('num cannot be greater than length')

    # subtract 1 from length to make it equivalent to the last index,
    # subtract 1 from num to make the calculation include the last index
    step = (length - 1) / (num - 1)

    return (int(round_(step * i)) for i in range(num))


def evenly_spaced_values(num, data):
    """
    Returns `num` evenly spaced values from sequence `data`.

    Parameters
    ----------
    num : int
    data : sequence

    Returns
    -------
    values : list

    """
    return [data[i] for i in n_to_indices(num, len(data))]


def make_name_map(names):
    """
    Create a dictionary mapping lowercase names to capitalized names.

    Parameters
    ----------
    names : sequence

    Returns
    -------
    dict

    """
    return dict((name.lower(), name) for name in names)


def make_names_and_lengths(names, lengths=None):
    """
    Create a list pairing palette names with lengths. (Mostly used to define
    the set of palettes that are automatically built.)

    Parameters
    ----------
    names : sequence of str
    lengths : sequence of int, optional

    Returns
    -------
    list of tuple
        Pairs of names and lengths.

    """
    lengths = lengths or range(3, 21)
    return list(itertools.product(names, lengths))


def palette_name(name, length):
    """Create a palette name like CubeYF_8"""
    return '{0}_{1}'.format(name, length)


def split_name_length(name):
    """Split name and length from a name like CubeYF_8"""
    split = name.split('_')
    return split[0], int(split[1])


def print_maps_factory(desc, names_and_lengths, palette_type):
    """
    Create a function that will print the names and lengths of palettes.

    Parameters
    ----------
    desc : str
        Short description of palettes, for example "sequential cmocean".
        Used to populate the print_maps docstring.
    names_and_lengths : sequence of tuple
        Pairs of names and lengths.
    palette_type : str
        Palette type to include in printed messages.

    Returns
    -------
    function
        Takes no arguments.

    """
    def print_maps():
        namelen = max(
            len(palette_name(name, length))
            for name, length in names_and_lengths)
        fmt = '{0:' + str(namelen + 4) + '}{1:16}{2:}'

        for name, length in names_and_lengths:
            print(fmt.format(
                palette_name(name, length), palette_type, length))
    print_maps.__doc__ = 'Print a list of {0} palettes'.format(desc)
    return print_maps


def get_map_factory(desc, mod_path, names_to_data, palette_type, palette_class,
                    is_evenly_spaced=True):
    """
    Create a function that builds a Palette instance from available data
    and the given Palette sub-class.

    Parameters
    ----------
    desc : str
        Short description of palettes, for example "sequential cmocean".
        Used to populate the get_map docstring.
    mod_path : str
        Path to module where get_map will be used. Use to point users to
        the print_maps function. E.g. 'palettable.cmocean.sequential'.
    names_to_data : dict
        Dictionary mapping string palette names to color data.
        (Lists of 0-255 integer RGB tuples.)
    palette_type : str
        Palette type to pass into Palette subclass, e.g. 'diverging'.
    palette_class : Palette subclass
        Subclass of Palette to use when creating new Palettes.
        Must have __init__ signature (name, palette_type, colors).
    is_evenly_spaced : bool
        Sets sampling of palette. If True, then choose values evenly spaced
        in palette array, otherwise choose colors from the beginning of the
        array.

    Returns
    -------
    function
        The function will have the definition:

            def get_map(name, reverse=False):

    """
    name_map = make_name_map(names_to_data.keys())

    def get_map(name, reverse=False):
        # name will be something like Viridis_8
        name, length = split_name_length(name)
        name_lower = name.lower()

        if name_lower not in name_map:
            raise KeyError('Unknown palette name: {0}'.format(name))

        name = name_map[name_lower]

        if len(names_to_data[name]) < length:
            raise ValueError('Number of requested colors larger than '
                             'the number available in {0}'.format(name))
        if is_evenly_spaced:
            colors = evenly_spaced_values(length, names_to_data[name])
        else:
            colors = names_to_data[name][:length]

        # add number back to name
        name = palette_name(name, length)

        if reverse:
            name += '_r'
            colors = list(reversed(colors))

        return palette_class(name, palette_type, colors)

    get_map.__doc__ = textwrap.dedent("""\
        Get a {0} palette by name.

        Parameters
        ----------
        name : str
            Name of map. Use {1}.print_maps
            to see available names.
        reverse : bool, optional
            If True reverse colors from their default order.

        Returns
        -------
        {2}

        """).format(desc, mod_path, palette_class.__class__.__name__)

    return get_map


def load_all_palettes(names_and_lengths, get_map_func):
    """
    Load all palettes (including reversed ones) into a dictionary,
    given a dictionary of names and lengths, plus a get_map function.

    Parameters
    ----------
    names_and_lengths : dict
        names_and_lengths : sequence of tuple
        Pairs of names and lengths.
    get_map_func : function
        Function with signature get_map(name, reverse=False) that returns
        a Palette subclass instance.

    Returns
    -------
    dict
        Maps Name_Length strings to instances of a Palette subclass.

    """
    maps = {}
    for name, length in names_and_lengths:
        map_name = palette_name(name, length)
        maps[map_name] = get_map_func(map_name)
        maps[map_name + '_r'] = get_map_func(map_name, reverse=True)
    return maps



"""
Qualitative color maps from the CartoColors schemes:
https://github.com/CartoDB/CartoColor/wiki/CARTOColor-Scheme-Names
"""
from __future__ import absolute_import

import itertools

from . import cartocolorspalette
from . import colormaps
from .. import utils

_PALETTE_TYPE = 'qualitative'
_NAMES_TO_DATA = {
    'Antique': colormaps._ANTIQUE,
    'Bold': colormaps._BOLD,
    'Pastel': colormaps._PASTEL,
    'Prism': colormaps._PRISM,
    'Safe': colormaps._SAFE,
    'Vivid': colormaps._VIVID,
}

_NAME_MAP = utils.make_name_map(_NAMES_TO_DATA.keys())
_NAMES_AND_LENGTHS = utils.make_names_and_lengths(sorted(_NAMES_TO_DATA.keys()),
                                                  range(2, 11))


print_maps = utils.print_maps_factory('qualitative cartocolors',
                                      _NAMES_AND_LENGTHS,
                                      _PALETTE_TYPE)

get_map = utils.get_map_factory('qualitative cartocolors',
                                __name__,
                                _NAMES_TO_DATA,
                                _PALETTE_TYPE,
                                cartocolorspalette.CartoColorsMap,
                                is_evenly_spaced=False)


globals().update(utils.load_all_palettes(_NAMES_AND_LENGTHS, get_map))
